/* Copyright (c) 2006-2018 Peter Speck / Vitality. All rights reserved.
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of Vitality nor the names of its contributors may be
 *       used to endorse or promote products derived from this software without
 *       specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY PETER SPECK / VITALITY AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL PETER SPECK / VITALITY AND CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR  SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package org.jabhts.junit.tests.sSplitTemplate;

import org.jabhts.annotation.attribute.content.AStyle;
import org.jabhts.annotation.element.EReplace;
import org.jabhts.annotation.element.EText;
import org.jabhts.annotation.global.TCompile;
import org.jabhts.junit.lib.TestBase;

public class SplitTemplate extends TestBase
{
	@TCompile(filename = "SplitTemplate", tpname = "SplitTemplatePageTp")
	class Page
	{
		@EText(tag = "title")
		String pageTitle = "The split test title";

		@EText(tag = "h2")
		String header = "This is the heading";

		@EReplace
		void makeSharedPart() throws Exception
		{
			pw.println("<!-- pre SplitTemplateSharedTp -->");
			new SplitTemplateSharedTp(SplitTemplate.this).emit(pw);
			pw.println("<!-- post SplitTemplateSharedTp -->");
		}

		@EReplace
		void makeReplaceMe() throws Exception
		{
			pw.println("<!-- pre SplitTemplateReplacedTp -->");
			new SplitTemplateReplacedTp(SplitTemplate.this).emit(pw);
			pw.println("<!-- post SplitTemplateReplacedTp -->");
		}
	}

	protected void emitTemplate() throws Exception
	{
		new SplitTemplatePageTp(this).emit(pw);
	}

	@TCompile(filename = "SplitTemplate", tpname = "SplitTemplateSharedTp", fragment = "SharedPart")
	static class SplitTemplateShared
	{
		@EText
		String name1 = "Hilmar Örn Hilmarsson";

		@EText
		String name2 = "Herbie Hancock";

		@EText
		String name3 = "Frankie Sparo";

		@EText
		String name4 = "Faun Fables";
	}

	@TCompile(filename = "SplitTemplate", tpname = "SplitTemplateReplacedTp", fragment = "ReplaceMe")
	static class SplitTemplateReplaced
	{
		@AStyle
		String adjective = "font-weight: bold;";

		@AStyle
		String replaceme = "color: green;";
	}

}
