/* Copyright (c) 2006-2018 Peter Speck / Vitality. All rights reserved.
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of Vitality nor the names of its contributors may be
 *       used to endorse or promote products derived from this software without
 *       specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY PETER SPECK / VITALITY AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL PETER SPECK / VITALITY AND CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR  SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package org.jabhts.runtime.html;

import java.util.Collections;
import java.util.HashSet;
import java.util.Set;
import org.w3c.dom.Element;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;

public class HtmlTags
{
	public static final Set<String> emptyElements;
	public static final Set<String> emptyAttributes;
	public static final Set<String> elementsPrependNewline;
	public static final Set<String> elementsAppendNewline;

	static {
		emptyElements = makeSet(new String[]{
				"area", "base", "basefont", "br", "col", "frame", "hr", "img",
				"input", "isindex", "link", "meta", "param"});
		//
		emptyAttributes = makeSet(new String[]{
				"checked", "compact", "declare", "defer", "disabled", "ismap",
				"nohref", "noresize", "noshade", "nowrap", "readonly", "selected"});
		//
		Set<String> set = new HashSet<>();
		for (String s : new String[]{
				"area", "base", "basefont", "body", "col", "div", "form", "frame",
				"h1", "h2", "h3", "h4", "h5", "h6",
				"head", "hr", "link", "meta", "param", "p", "script", "style",
				"table", "tbody", "td", "title", "th", "tr"}) {
			set.add(s);
		}
		elementsPrependNewline = Collections.unmodifiableSet(new HashSet<>(set));
		set.add("br");
		set.add("html");
		elementsAppendNewline = Collections.unmodifiableSet(set);
	}

	private static Set<String> makeSet(String[] a)
	{
		Set<String> set = new HashSet<>();
		for (String s : a)
			set.add(s);
		return Collections.unmodifiableSet(set);
	}

	public static String elementToString(Element e)
	{
		StringBuilder sb = new StringBuilder(10);
		sb.append('<').append(e.getNodeName());
		NamedNodeMap map = e.getAttributes();
		int num = map.getLength();
		for (int i = 0; i < num; i++) {
			Node a = map.item(i);
			sb.append(' ').append(a.getNodeName());
			if (emptyElements.contains(a.getNodeName()))
				continue;
			sb.append('=');
			sb.append(HtmlAttr.quoted(a.getNodeValue()));
		}
		sb.append('>');
		return sb.toString();
	}
}
