/* Copyright (c) 2006-2018 Peter Speck / Vitality. All rights reserved.
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of Vitality nor the names of its contributors may be
 *       used to endorse or promote products derived from this software without
 *       specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY PETER SPECK / VITALITY AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL PETER SPECK / VITALITY AND CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR  SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package org.jabhts.translator.compiler.application.baseclass;

import org.jabhts.translator.compiler.application.attribute.AttributeEmitter;
import org.jabhts.translator.compiler.application.scope.TargetingInfo;
import org.jabhts.translator.compiler.codeGeneration.api.PatchedGoto;
import org.jabhts.translator.compiler.codeGeneration.api.Program;
import org.jabhts.translator.compiler.utils.CompilerException;
import org.jabhts.translator.compiler.utils.InternalCompilerException;
import org.jabhts.translator.compiler.utils.LogLevel;

public abstract class AbstractStringApplication extends AbstractValueApplication
{
	public enum EncodingType
	{
		Raw, Html, Attr
	}

	protected EncodingType encoding;

	protected AbstractStringApplication(TargetingInfo target, EncodingType encoding) throws CompilerException
	{
		super(target, true, String.class);
		this.encoding = encoding;
		if (encoding != EncodingType.Raw && callbackType == CallbackType.VoidMethod) {
			log.log(LogLevel.Error, "Can't use @" + target.annotation.annotationType().getSimpleName()
					+ " on void method: " + annotationToString());
		}
	}

	protected void generate(AttributeEmitter<AbstractStringApplication> emitter) throws CompilerException
	{
		Program program = emitter.program;
		emitStringCallback(program);
		program.addTestIsStringNull();
		PatchedGoto pg = program.addPatchedConditionalGoto();
		printStringResult(program);
		pg.patchToCurrentAddress();
	}

	public void emitStringCallbackWithPrint(Program program) throws CompilerException
	{
		switch (callbackType) {
			case Field:
				program.addGetStringField(scope, field);
				printStringResult(program);
				break;
			case Method:
				program.addCallback(scope, method);
				printStringResult(program);
				break;
			case VoidMethod:
				program.addCallback(scope, method);
				break;
			default:
				throw new CompilerException("Unsupported callbackType = " + callbackType + ": " + annotationToString());
		}
	}

	public void emitStringCallback(Program program) throws CompilerException
	{
		switch (callbackType) {
			case Field:
				program.addGetStringField(scope, field);
				break;
			case Method:
				program.addCallback(scope, method);
				break;
			case VoidMethod:
				program.addCallback(scope, method);
				break;
			default:
				throw new CompilerException("Unsupported callbackType = " + callbackType + ": " + annotationToString());
		}
	}

	public void printStringResult(Program program) throws CompilerException
	{
		switch (encoding) {
			case Raw:
				program.addPrintRawStringResult();
				break;
			case Html:
				program.addPrintHtmlEncodedStringResult();
				break;
			case Attr:
				program.addPrintAttrEncodedStringResult();
				break;
			default:
				throw new InternalCompilerException("invalid encoded = " + encoding);
		}
	}
}
