/* Copyright (c) 2018 Peter Speck / Vitality. All rights reserved.
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of Vitality nor the names of its contributors may be
 *       used to endorse or promote products derived from this software without
 *       specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY PETER SPECK / VITALITY AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL PETER SPECK / VITALITY AND CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR  SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package org.jabhts.translator.compiler.htmlLoader;

import org.jabhts.translator.compiler.utils.CompilerException;
import org.w3c.dom.Element;
import org.w3c.dom.Node;

public class HtmlTransformerMetaTags implements HtmlTransformer
{
	public void transform(HtmlDocument hdoc) throws Exception
	{
		for (Node head = hdoc.doc.getDocumentElement().getFirstChild(); head != null; head = head.getNextSibling()) {
			if (head.getNodeType() != Node.ELEMENT_NODE || !head.getNodeName().equalsIgnoreCase("head"))
				continue;
			Node next;
			for (Node meta = head.getFirstChild(); meta != null; meta = next) {
				next = meta.getNextSibling();
				if (meta.getNodeType() != Node.ELEMENT_NODE || !meta.getNodeName().equalsIgnoreCase("meta"))
					continue;
				Element e = (Element)meta;
				if (shouldZapMetaElement(e))
					head.removeChild(meta);
			}
		}
	}

	protected boolean shouldZapMetaElement(Element e) throws CompilerException
	{
		return isHtml5CharsetMeta(e)
				|| e.getAttribute("name").equalsIgnoreCase("generator")
				|| e.getAttribute("http-equiv").equalsIgnoreCase("Content-Type");

	}

	protected boolean isHtml5CharsetMeta(Element e)
	{
		return !e.getAttribute("charset").isEmpty()
				&& e.getAttribute("name").isEmpty()
				&& e.getAttribute("http-equiv").isEmpty();
	}
}
