/* Copyright (c) 2006-2011, Peter Speck / Vitality. All rights reserved.
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of Vitality nor the names of its contributors may be
 *       used to endorse or promote products derived from this software without
 *       specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY PETER SPECK / VITALITY AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL PETER SPECK / VITALITY AND CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR  SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package dk.vitality.lib.foundation.misc;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;
import org.jetbrains.annotations.NotNull;

public class MD5Utils
{
	@NotNull
	public static String md5LcHex(@NotNull byte[] bytes) throws NoSuchAlgorithmException
	{
		return HexUtils.toLcHex(MessageDigest.getInstance("MD5").digest(bytes));
	}

	@NotNull
	public static String md5LcHex(@NotNull MessageDigest digest)
	{
		return HexUtils.toLcHex(digest);
	}

	@NotNull
	public static String md5LcHex(@NotNull String s) throws NoSuchAlgorithmException, UnsupportedEncodingException
	{
		return md5LcHex(s.getBytes("utf-8"));
	}

	@NotNull
	public static String getFileDataMD5(@NotNull File f) throws IOException, NoSuchAlgorithmException
	{
		if (f.length() == 0)
			return HexUtils.toLcHex(ArrayUtils.emptyByteArray);
		int sz = (int)Math.min(maxBufferSize, f.length());
		byte[] buffer = new byte[sz];
		return getFileDataMD5(f, buffer);
	}

	@NotNull
	public static String getFileDataMD5(@NotNull File f, @NotNull byte[] buffer) throws IOException, NoSuchAlgorithmException
	{
		try (FileInputStream fis = new FileInputStream(f)) {
			MessageDigest md5 = MessageDigest.getInstance("MD5");
			int loops = 0;
			while (true) {
				int num = fis.read(buffer);
				if (num < 0)
					break;
				md5.update(buffer, 0, num);
				if (loops++ > 16 && num == buffer.length && buffer.length < maxBufferSize) {
					int sz = buffer.length * 4;
					System.err.println("Reallocs buffer in MD5Utils (" + buffer.length + " -> " + sz + ")" +
							", file size now " + f.length());
					buffer = new byte[sz]; // in case file grows while reading it.
				}
			}
			return HexUtils.toLcHex(md5);
		}
	}

	private static final int maxBufferSize = 256 * 1024;
}
