/* Copyright (C) 2008-2015 Peter Speck
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#import <Cocoa/Cocoa.h>

@class Prefs;
@class AlertHelper;
@class XmlDoc;

@interface ServerCommunication : NSObject {
}

@property (readonly) NSBundle* mainBundle;
@property (readonly) Prefs* prefs;
@property (readonly) NSString* publicAdminKey;

- (instancetype)initWithPrefs:(Prefs*)prefs
			   withMainBundle:(NSBundle*)mainBundle;
- (void)performPostLoadPreferencePanelCheck;


- (void)showServerErrorAlertWithUrl:(NSURL*)url
						  withError:(__autoreleasing NSError*)error;

#pragma mark ------------------------------------- URLs

- (NSString*)getAdminUrl:(NSString*)pathArgs;

#pragma mark ------------------------------------- updater

- (void)updateUpdaterLaunchd;
- (void)launchUpdaterApp;

#pragma mark ------------------------------------- server management

- (BOOL)isServerRunning;
- (void)startServer;
- (void)stopServer;
- (void)openServerLogInConsole;

- (int)getPortNumber;

- (void)sendReloadPreferencesForModifiedFilter:(int)filterId withRule:(int)ruleId;
- (void)saveDirtyPrefsAndNotify;

#pragma mark ------------------------------------- curl, NSData

// If withErrorRef is NULL, an alert sheet is displayed for errors.

- (NSData*)curlToData:(NSString*)pathArgs
		 withResponse:(NSURLResponse**)response
		 withErrorRef:(__autoreleasing NSError**)errorRef;

#pragma mark ------------------------------------- curl, NSString

- (NSString*)curl:(NSString*)pathArgs;
- (NSString*)curl:(NSString*)pathArgs
	 withErrorRef:(__autoreleasing NSError**)errorRef;

#pragma mark ------------------------------------- curl, XmlDoc

- (XmlDoc*)curlToXml:(NSString*)pathArgs withActionDescription:(NSString*)actionDesc;
- (XmlDoc*)curlToXml:(NSString*)pathArgs withActionDescription:(NSString*)actionDesc
		withErrorRef:(__autoreleasing NSError**)errorRef;

@end
