/* Copyright (C) 2008-2015 Peter Speck
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package org.glimmerblocker.proxy.admin.filter;

import java.util.LinkedHashMap;
import java.util.Map;
import org.glimmerblocker.proxy.admin.AbstractAdminXmlHandler;
import org.glimmerblocker.proxy.globals.subscriptions.FilterNetworkData;
import org.glimmerblocker.proxy.proxyRequest.ProxyRequestHandler;
import org.jetbrains.annotations.NotNull;
import dk.vitality.lib.foundation.exception.UserErrorException;

public class AdminFilterSubAdd extends AbstractAdminXmlHandler
{
	public AdminFilterSubAdd(@NotNull ProxyRequestHandler req) throws Exception
	{
		super(req);
	}

	/*
		Need a 2-step process so the user can cancel the update dialog without having
		the subscription added when the server finally answers.
		So first step is to get the xml and store it in a LRU cache.
		Second step (which is immediate) stores the data into the subscription "database".
	 */
	protected void processAdminXml() throws Exception
	{
		String url = urlParams.reqString("url");
		int filterId = urlParams.reqInt("filter-id");
		if (globals.panelSettingsXA.exists("filters/filter[@id='" + filterId + "']"))
			throw new UserErrorException("Internal error: duplicate filter-id.");
		if (urlParams.has("commit")) {
			FilterNetworkData[] a;
			synchronized (cache) {
				a = cache.get(filterId);
				if (a == null || a[0] == null)
					throw new Exception("No such subscription being added.");
				globals.filterSubscriptions.saveFilterData(a[0]);
			}
			rootE.appendChild(doc.importNode(a[0].docXA.getContextElement(), true));
			return;
		}
		FilterNetworkData[] a = new FilterNetworkData[1];
		synchronized (cache) {
			cache.put(filterId, a);
		}
		a[0] = new FilterNetworkData(globals, this, filterId, url, null, true);
		if (a[0].errorMessage != null)
			throw new UserErrorException(a[0].errorMessage);
	}

	//===============================================================

	private static final LruCache cache = new LruCache();

	private static class LruCache extends LinkedHashMap<Integer, FilterNetworkData[]>
	{
		// use array as poor mans reference.

		private LruCache()
		{
			super(5);
		}

		@Override
		protected boolean removeEldestEntry(Map.Entry<Integer, FilterNetworkData[]> eldest)
		{
			return size() > 5;
		}
	}
}
