/* Copyright (C) 2008-2015 Peter Speck
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package org.glimmerblocker.proxy.admin.misc;

import java.util.Collections;
import java.util.List;
import org.glimmerblocker.proxy.admin.AbstractAdminXmlHandler;
import org.glimmerblocker.proxy.globals.suspect.Suspect;
import org.glimmerblocker.proxy.globals.suspect.SuspectType;
import org.glimmerblocker.proxy.proxyRequest.ProxyRequestHandler;
import org.jetbrains.annotations.NotNull;
import org.w3c.dom.Element;

public class AdminSuspectList extends AbstractAdminXmlHandler
{
	public AdminSuspectList(@NotNull ProxyRequestHandler req) throws Exception
	{
		super(req);
	}

	protected void processAdminXml() throws Exception
	{
		boolean js = urlParams.optBoolean("js", true);
		List<Suspect> list = globals.suspects.getSuspects();
		Collections.reverse(list); // newest is at end, and we want them at the start.
		for (Suspect suspect : list) {
			if (!js && suspect.type == SuspectType.Javascript)
				continue;
			Element e = doc.createElement("suspect");
			setAttr(e, "host", suspect.host);
			setAttr(e, "path", suspect.path);
			setAttr(e, "query", suspect.query);
			setAttr(e, "referer", suspect.referer);
			//setAttr(e, "type", suspect.type.name());
			setAttr(e, "why", suspect.description);
			if (suspect.crossDomain)
				setAttr(e, "cross-domain", "1");
			rootE.appendChild(e);
		}
	}
}