/* Copyright (C) 2009-2015 Peter Speck
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package org.glimmerblocker.proxy.globals.network;

import java.io.IOException;
import org.glimmerblocker.proxy.globals.GlimmerGlobals;
import org.glimmerblocker.proxy.globals.log.Logger;
import org.glimmerblocker.proxy.rhino.callback.CallbackRhinoBinder;
import org.glimmerblocker.proxy.rhino.callback.GlimmerGlobalObjectBase;
import org.glimmerblocker.proxy.rhino.callback.RhinoCallback;
import org.jetbrains.annotations.NotNull;
import org.mozilla.javascript.Context;
import org.mozilla.javascript.Scriptable;
import dk.vitality.lib.foundation.ref.FatRef;

public class NetworkConfigGlobalObject extends GlimmerGlobalObjectBase
{
	FatRef<NetworkConfig> ref;

	NetworkConfigSubObject network;

	private static final CallbackRhinoBinder callbackBinder = new CallbackRhinoBinder(NetworkConfigGlobalObject.class);

	NetworkConfigGlobalObject(@NotNull GlimmerGlobals globals,
	                          @NotNull Scriptable scope,
	                          @NotNull Scriptable prototype) throws IOException
	{
		this(globals, scope, prototype, callbackBinder);
	}

	public NetworkConfigGlobalObject(@NotNull GlimmerGlobals globals, @NotNull Scriptable scope,
	                                 @NotNull Scriptable prototype,
	                                 @NotNull CallbackRhinoBinder binder) throws IOException
	{
		super(globals, scope, prototype, binder);
	}

	public void prepareNetworkConfigCallback(@NotNull Logger logger,
	                                         @NotNull Context ctx,
	                                         @NotNull FatRef<NetworkConfig> ref) throws Exception
	{
		super.prepareGlobalBase(logger, ctx);
		this.ref = ref;
		if (network != null)
			network.prepareNetworkConfigSubObject(ctx, ref);
	}

	@Override
	public void notifyLeftContext()
	{
		super.notifyLeftContext();
		ref = null;
	}

	@RhinoCallback
	public NetworkConfigSubObject jsGet_network() throws Exception
	{
		if (network == null) {
			network = new NetworkConfigSubObject(this);
			network.bind();
			network.prepareNetworkConfigSubObject(ctx, ref);
		}
		return network;
	}
}
