/* Copyright (C) 2008-2015 Peter Speck
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package org.glimmerblocker.proxy.http.client.connection;

import java.io.IOException;
import java.io.OutputStream;
import org.jetbrains.annotations.NotNull;

public class ChunkedOutputStream extends OutputStream
{
	@NotNull final OutputStream os;
	private boolean closed;

	public ChunkedOutputStream(@NotNull OutputStream os)
	{
		this.os = os;
	}

	public void write(int b) throws IOException
	{
		throw new IOException("Single-byte writes are stupid. Use a BufferedOutputStream.");
	}

	@Override
	public void write(@NotNull byte[] data, int offset, int len) throws IOException
	{
		if (closed)
			throw new IOException("OutputStream is closed");
		if (len < 0)
			throw new IllegalArgumentException("len < 0");
		if (len == 0)
			return;
		String s = Integer.toHexString(len);
		os.write(s.getBytes("ASCII"));
		os.write(13);
		os.write(10);
		os.write(data, offset, len);
		os.write(13);
		os.write(10);
	}

	@Override
	public void close() throws IOException
	{
		if (closed)
			return;
		closed = true;
		// zero chunk-size means end of content.
		os.write('0');
		os.write(13);
		os.write(10);
		// no trailer.
		os.write(13); // write post-trailer empty line.
		os.write(10);
		os.flush();  // flush() member is a no-op at this point.
		notifyHasSentFullResponseBody();
	}

	protected void notifyHasSentFullResponseBody()
	{
	}

	@Override
	public void flush() throws IOException
	{
		if (closed)
			return;
		os.flush();
	}
}
