/* Copyright (C) 2009-2015 Peter Speck
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package org.glimmerblocker.proxy.http.client.curl;

import java.net.InetAddress;
import java.net.URL;
import org.glimmerblocker.proxy.http.data.Headers;
import org.glimmerblocker.proxy.rhino.callback.CallbackObjectBase;
import org.glimmerblocker.proxy.rhino.callback.CallbackRhinoBinder;
import org.glimmerblocker.proxy.rhino.callback.CallbackSubObject;
import org.glimmerblocker.proxy.rhino.callback.RhinoCallback;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;
import org.mozilla.javascript.Context;

public class StaticRequestSubObject extends CallbackSubObject
{
	public InetAddress clientAddr;
	public String method;
	public URL url;
	public Headers origHeaders;
	public HeadersMapObject headers;

	private static final CallbackRhinoBinder classBinder = new CallbackRhinoBinder(StaticRequestSubObject.class);

	public StaticRequestSubObject(@NotNull CallbackObjectBase parent)
	{
		super(parent, classBinder);
	}

	public StaticRequestSubObject(@NotNull CallbackObjectBase parent, @NotNull CallbackRhinoBinder binder)
	{
		super(parent, binder);
	}

	public void prepareStaticRequestSubObject(@NotNull Context ctx,
	                                          @Nullable InetAddress clientAddr,
	                                          @NotNull String method,
	                                          @NotNull URL url,
	                                          @NotNull Headers origHeaders) throws Exception
	{
		prepareObjectBase(ctx);
		this.clientAddr = clientAddr;
		this.method = method;
		this.url = url;
		this.origHeaders = origHeaders;
		this.headers = null;
	}

	@Override
	public void notifyLeftContext()
	{
		super.notifyLeftContext();
		this.method = null;
		this.url = null;
		this.origHeaders = null;
		this.headers = null;
	}

	@RhinoCallback
	public HeadersMapObject jsGet_headers()
	{
		if (headers != null)
			return headers;
		checkInsideScope();
		headers = new HeadersMapObject(this, ctx, scope, origHeaders);
		sealHeadersMapObject();
		return headers;
	}

	protected void sealHeadersMapObject()
	{
		headers.sealObject();
	}

	//===========================================================================

	@RhinoCallback
	public String jsGet_clientAddr()
	{
		return clientAddr.getHostAddress();
	}

	@RhinoCallback
	public String jsGet_clientHostname()
	{
		return clientAddr.getHostName();
	}

	//===========================================================================

	@RhinoCallback
	public String jsGet_method()
	{
		return method;
	}

	//===========================================================================

	@RhinoCallback
	public String jsGet_url()
	{
		return url.toString();
	}

	@RhinoCallback
	public String jsGet_urlPath()
	{
		String s = url.getPath();
		return (s != null && s.length() > 0) ? s : "/";
	}

	@RhinoCallback
	public String jsGet_urlHost()
	{
		return url.getHost();
	}

	@RhinoCallback
	public int jsGet_urlPort()
	{
		int p = url.getPort();
		if (p == 0 && url.getProtocol().equals("http"))
			p = 80;
		else 		if (p == 0 && url.getProtocol().equals("https"))
			p = 443;
		return p;
	}

	@RhinoCallback
	public String jsGet_urlHostPort()
	{
		return jsGet_urlHost() + ":" + jsGet_urlPort();
	}

	@RhinoCallback
	public String jsGet_urlQuery()
	{
		String s = url.getQuery();
		return s != null ? s : "";
	}

	//===========================================================================

	@Nullable
	@RhinoCallback
	public String jsGet_referer()
	{
		return jsGet_headers().getHeaderLC("referer");
	}

	//===========================================================================

	@Nullable
	@RhinoCallback
	public String jsGet_userAgent()
	{
		return jsGet_headers().getHeaderLC("user-agent");
	}

	//===========================================================================

	@Nullable
	@RhinoCallback
	public String jsGet_contentType()
	{
		return jsGet_headers().getHeaderLC("content-type");
	}

	@Nullable
	@RhinoCallback
	public String jsGet_baseContentType()
	{
		return jsGet_headers().getBaseContentType();
	}
}
