/* Copyright (C) 2008-2015 Peter Speck
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package org.glimmerblocker.proxy.http.client.stream;

import java.io.InputStream;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

public class RequestBody
{
	@NotNull public static final RequestBody NONE = new RequestBody(null, -1);

	@Nullable public final InputStream stream;

	/**
	 * A 'content-length' header is sent if the value is &gt;= 0.
	 * Specify -1 to omit the content-length header for e.g. GET requests
	 * or Transfer-Encoding=chunked.
	 */
	public final long contentSize;

	public final boolean isEmpty;

	public final boolean hasUnknownSize;

	/**
	 * @param contentSize A 'content-length' header is sent if the value is &gt;= 0.
	 *                    Specify -1 to omit the content-length header for e.g. GET requests.
	 */
	public RequestBody(@Nullable InputStream stream, long contentSize)
	{
		this.stream = stream;
		this.contentSize = contentSize;
		if (stream == null) {
			if (contentSize > 0)
				throw new IllegalArgumentException("Must have bodyStream when request-content-length > 0");
			isEmpty = true;
			hasUnknownSize = false;
			return;
		}
		if (stream instanceof DevNullInputStream) {
			if (contentSize > 0)
				throw new IllegalArgumentException("DevNullInputStream with content-length = " + contentSize + " > 0");
			isEmpty = true;
			hasUnknownSize = false;
			return;
		}
		isEmpty = (contentSize == 0);
		hasUnknownSize = (contentSize < 0);
	}
}
