/* Copyright (C) 2008-2015 Peter Speck
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package org.glimmerblocker.proxy.proxyRequest.transform;

import java.net.HttpURLConnection;
import org.jetbrains.annotations.Nullable;

public enum MimeTypeMatcher
{
	Html
			{
				@SuppressWarnings({"SimplifiableIfStatement"})
				public boolean matches(int httpStatus, String ctLC)
				{
					if (httpStatus != HttpURLConnection.HTTP_OK)
						return false;
					return ctLC.equals("text/html")
							|| ctLC.startsWith("text/xhtml")
							|| ctLC.startsWith("application/xhtml");
				}
			},
	Xml
			{
				@SuppressWarnings({"SimplifiableIfStatement"})
				public boolean matches(int httpStatus, String ctLC)
				{
					if (httpStatus != HttpURLConnection.HTTP_OK)
						return false;
					if (ctLC.equals("application/smil"))
						return true;
					return (ctLC.contains("/xml") || ctLC.endsWith("+xml")) && !Html.matches(httpStatus, ctLC);
				}
			},
	Css
			{
				@SuppressWarnings({"SimplifiableIfStatement"})
				public boolean matches(int httpStatus, String ctLC)
				{
					if (httpStatus != HttpURLConnection.HTTP_OK)
						return false;
					return ctLC.equals("text/css");
				}
			},
	Js
			{
				@SuppressWarnings({"SimplifiableIfStatement"})
				public boolean matches(int httpStatus, String ctLC)
				{
					if (httpStatus != HttpURLConnection.HTTP_OK)
						return false;
					return ctLC.equals("application/x-javascript")
							|| ctLC.equals("application/javascript")
							|| ctLC.equals("text/javascript")
							|| ctLC.equals("text/ecmascript");
				}
			},
	Json
			{
				@SuppressWarnings({"SimplifiableIfStatement"})
				public boolean matches(int httpStatus, String ctLC)
				{
					if (httpStatus != HttpURLConnection.HTTP_OK)
						return false;
					return ctLC.equals("application/json");
				}
			},
	AnyTextType
			{
				@SuppressWarnings({"SimplifiableIfStatement"})
				public boolean matches(int httpStatus, String ctLC)
				{
					if (httpStatus != HttpURLConnection.HTTP_OK)
						return false;
					return ctLC.startsWith("text/")
							|| Js.matches(httpStatus, ctLC)
							|| Html.matches(httpStatus, ctLC)
							|| ctLC.equals("application/json")
							;
				}
			},
	ErrorStatusType
			{
				public boolean matches(int httpStatus, String ctLC)
				{
					return httpStatus >= 300; // don't include 100-299.
				}
			};

	public abstract boolean matches(int httpStatus, String ctLC);

	public static MimeTypeMatcher fromXml(@Nullable String s) throws Exception
	{
		if (s == null || s.equals("html"))
			return Html;
		if (s.equals("xml"))
			return Xml;
		if (s.equals("css"))
			return Css;
		if (s.equals("js"))
			return Js;
		if (s.equals("json"))
			return Json;
		if (s.equals("*"))
			return AnyTextType;
		if (s.equals("*not-ok*"))
			return ErrorStatusType;
		throw new Exception("Invalid rule.content-type='" + s + "'");
	}
}
