/* Copyright (C) 2008-2015 Peter Speck
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package org.glimmerblocker.proxy.proxyRequest.transform;

import java.io.StringWriter;
import java.util.List;
import java.util.regex.Pattern;
import org.jabhts.annotation.element.EHtml;
import org.jabhts.annotation.element.EText;
import org.jabhts.annotation.flowControl.EIf;
import org.jabhts.annotation.flowControl.EIterate;
import org.jabhts.annotation.global.TCompile;
import org.jabhts.runtime.html.HtmlPrintWriter;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;
import dk.vitality.lib.foundation.misc.RandomUtils;
import dk.vitality.lib.foundation.misc.StringUtils;

@TCompile(fragmentChilds = true)
public class TransformErrorReport
{
	@NotNull private final StringWriter sw;
	@NotNull private final HtmlPrintWriter pw;
	@NotNull private final List<TransformErrorInfo> errorList;

	public TransformErrorReport(@NotNull List<TransformErrorInfo> errorList) throws Exception
	{
		this.errorList = errorList;
		sw = new StringWriter(1000);
		pw = new HtmlPrintWriter(sw);
		new TransformErrorReportTp(this).emit(pw);
		pw.flush();
	}

	@Nullable
	public String getErrorHtml()
	{
		String errorHtml = sw.toString();
		// randomize the ID
		String id = "X" + RandomUtils.randomizer.nextLong();
		errorHtml = StringUtils.replaceAll(errorHtml, "gb_error_42", id);
		errorHtml = whitespaceRunsRE.matcher(errorHtml).replaceAll(" ");
		return "\n\n\n\n" +
				errorHtml +
				"\n\n\n\n";
	}

	private static final Pattern whitespaceRunsRE = Pattern.compile("(&#(?:9|10|13);)+");


	@EIterate
	List<TransformErrorInfo> iterErrorBox()
	{
		return errorList;
	}

	class EmitErrorBox
	{
		@NotNull final TransformErrorInfo info;

		EmitErrorBox(@NotNull TransformErrorInfo info)
		{
			this.info = info;
		}

		@EText
		String getErrorMessage()
		{
			return info.msg;
		}

		@EHtml
		void printRuleDescription()
		{
			pw.print("Filter: ");
			pw.htmlEncode(info.rule.filterName);
			pw.println("<br>");
			pw.htmlEncode(info.rule.hostMatchType.userDescription(info.rule.host));
			pw.println("<br>");
			pw.htmlEncode(info.rule.pathMatchType.userDescription(info.rule.path));
			pw.println("<br>");
			pw.htmlEncode(info.rule.queryMatchType.userDescription(info.rule.query));
		}

		@EIf({"ErrorLocation", "ErrorSource"})
		boolean hasErrorLocation()
		{
			return info.line > 0;
		}

		@EHtml
		void printErrorLocation()
		{
			pw.printf("Error in line %d, column %d", info.line, info.col);
		}

		@EHtml
		void printErrorSource()
		{
			String[] a = StringUtils.splitByChar(info.rule.transform, '\n');
			boolean first = true;
			for (int i = 0; i < 6; i++) {
				int idx = info.line - 3 + i;
				if (idx < 0 || idx >= a.length)
					continue;
				if (first)
					first = false;
				else
					pw.println(); // using <pre>
				pw.htmlEncode(a[idx]);
				if (i != 2)
					continue;
				pw.println(); // using <pre>
				for (int x = 1; x < info.col; x++)
					pw.print('.');
				pw.print("^");
				for (int x = info.col + 1; x < a[idx].length(); x++)
					pw.print('.');
			}
		}
	}
}