/* Copyright (C) 2008-2015 Peter Speck
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package org.glimmerblocker.proxy.rule;

import org.glimmerblocker.proxy.util.XmlParserHelper;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import dk.vitality.lib.foundation.misc.StringUtils;
import dk.vitality.lib.foundation.xml.xpath.XmlArgs;

public class RuleUnitTests
{
	public static void main(String[] argv)
	{
		try {
			tests();
		}
		catch (Exception ex) {
			System.err.println("Got exception: " + ex.getMessage());
			ex.printStackTrace();
		}
	}

	private static void tests() throws Exception
	{
		AbstractRule[] list = new AbstractRule[]{
				blocker("path=/;priority=5;mobile=1"),
				blocker("path=/;priority=5"),
				blocker("path=/abc;path-type=is"),
				blocker("path=/abc;host=doubleclick;mobile=1"),
				blocker("path=/abc;host=doubleclick"),
				transformer("path=/abc"),
				whitelist("path=/abc"),
				blocker("path=/abc"),
				whitelist("path=/w"),
				blocker("path=/w"),
				blocker("path=/d"),
				blocker("path=/;host=doubleclick;query=def"),
				blocker("path=/;host=doubleclick"),
				blocker("path=/;query=def"),
				whitelist("path=/"),
				blocker("path=/"),
				blocker("path=b;path-type=regexp"),
				blocker("path=bc;path-type=ends-with"),
				blocker("path=/;priority=1;mobile=1"),
				blocker("path=/;priority=1"),
		};
		for (int i = 0; i < list.length; i++) {
			AbstractRule r = list[i];
			int diff = r.compareTo(r);
			if (diff != 0) {
				ln("Got a-a diff = " + diff);
				ln("  " + i + ": " + r);
			}
		}
		// n^2, but we really want to test everything and not assume it just works; it's a black box.
		for (int a = 0; a + 1 < list.length; a++) {
			for (int b = a + 1; b < list.length; b++) {
				AbstractRule r1 = list[a];
				AbstractRule r2 = list[b];
				int diff = r1.compareTo(r2);
				if (diff >= 0) {
					ln("Got a-b diff = " + diff);
					ln("  " + a + ": " + r1);
					ln("  " + b + ": " + r2);
				}
				diff = r2.compareTo(r1);
				if (diff <= 0) {
					ln("Got b-a diff = " + diff);
					ln("  " + a + ": " + r1);
					ln("  " + b + ": " + r2);
				}
			}
		}
	}

	private static void ln(String s)
	{
		System.err.println(s);
	}

	private static WhitelistRule whitelist(String specification) throws Exception
	{
		return new WhitelistRule(false, 42, "UnitTests", mk(specification));
	}

	private static BlockerRule blocker(String specification) throws Exception
	{
		return new BlockerRule(false, 42, "UnitTests", mk(specification));
	}

	private static ModificationRule transformer(String specification) throws Exception
	{
		XmlArgs ruleXA = mk(specification);
		Element jsE = doc.createElement("js");
		ruleXA.getContextElement().appendChild(jsE);
		jsE.appendChild(doc.createTextNode("a = 42;"));
		return new ModificationRule(false, 42, "UnitTests", ruleXA);
	}

	private static XmlArgs mk(String specification) throws Exception
	{
		if (doc == null)
			doc = XmlParserHelper.createDocument();
		Element e = doc.createElement("rule");
		for (String s : StringUtils.splitByChar(specification, ';')) {
			s = s.trim();
			int idx = s.indexOf('=');
			if (idx <= 0)
				throw new Exception("Invalid specification (missing '='): " + specification);
			e.setAttribute(s.substring(0, idx), s.substring(idx + 1));
		}
		return XmlArgs.create(e);
	}

	private static Document doc;
}
